@@ -49,28 +49,21 @@
 
 //  -----------------------------------------------------------------
 //
-// System logging
+// System Logging or USB Prober Logging
 //
 //#define sleepTime 20
 #define sleepTime 0
 
-#ifdef DEBUGLOGGING
+#ifdef DEBUGLOGGING /* { */
+#ifdef CONSOLELOGGING /* { */
 #define debugIOLog( message... ) \
         do {IOLog( #message "\n", message ); IOSleep(sleepTime);} while (0)
-#else
+#else /* }{ */
+#define debugIOLog( message... ) \
+		do {USBLog( DEBUG_LEVEL_DEVELOPMENT, message );} while (0)
+#endif /* } */
+#else /* }{ */
 #define debugIOLog( message... ) ;
-#endif
-
-//
-// USB Prober logging
-//
-#if 0
-    #ifdef DEBUGLOGGING
-    #define debugIOLog( message... ) \
-            do {USBLog( DEBUG_LEVEL_DEVELOPMENT, message );} while (0)
-    #else
-    #define debugIOLog( message... ) ;
-    #endif
-#endif
+#endif /* } */
 
 #endif /* _APPLEUSBAUDIOCOMMON_H */
