@@ -142,6 +142,7 @@ IOReturn AppleUSBAudioDevice::protectedInitHardware (IOService * provider) {
 	debugIOLog ("+AppleUSBAudioDevice[%p]::start (%p)", this, provider);
 
 	resultCode = FALSE;
+	FailIf (NULL == mControlInterface, Exit);
 
 	debugIOLog ("There are %d configurations on this device", mControlInterface->GetDevice()->GetNumConfigurations ());
 	debugIOLog ("Our control interface number is %d", mControlInterface->GetInterfaceNumber ());
@@ -252,6 +253,7 @@ Boolean AppleUSBAudioDevice::ShouldUpdatePRAM (void) {
 	result = FALSE;
 	speakersGood = FALSE;
 	connectionGood = FALSE;
+	FailIf (NULL == mControlInterface, Exit);
 
 	// Make sure they're speakers that can support boot beep
 	vendorID = mControlInterface->GetDevice()->GetVendorID ();
@@ -428,6 +430,7 @@ IOReturn AppleUSBAudioDevice::doControlStuff (IOAudioEngine *audioEngine, UInt8
 
     usbAudioEngine = OSDynamicCast (AppleUSBAudioEngine, audioEngine);
     FailIf (NULL == usbAudioEngine, Exit);
+	FailIf (NULL == mControlInterface, Exit);
 	debugIOLog ("this usbAudioEngine = %p", usbAudioEngine);
 
 	if (NULL == mRegisteredEngines) {
@@ -598,6 +601,7 @@ IOReturn AppleUSBAudioDevice::doPlaythroughSetup (AppleUSBAudioEngine * usbAudio
 	IOReturn							result;
 
 	result = kIOReturnError;
+	FailIf (NULL == mControlInterface, Exit);
 
 	controlInterfaceNum = mControlInterface->GetInterfaceNumber ();
 
@@ -677,6 +681,7 @@ IOReturn AppleUSBAudioDevice::addSelectorSourcesToSelectorControl (IOAudioSelect
 	UInt8								selectorID;
 	UInt8								controlInterfaceNum;
 
+	FailIf (NULL == mControlInterface, Exit);
 	controlInterfaceNum = mControlInterface->GetInterfaceNumber ();
 
 	aPath = OSDynamicCast (OSArray, arrayOfPathsFromOutputTerminal->getObject (0));
@@ -715,6 +720,7 @@ OSString * AppleUSBAudioDevice::getNameForPath (OSArray * arrayOfPathsFromOutput
 
 	done = FALSE;
 	theString = NULL;
+	FailIf (NULL == mControlInterface, Exit);
 
 	aPath = OSDynamicCast (OSArray, arrayOfPathsFromOutputTerminal->getObject (*pathIndex));
 	FailIf (NULL == aPath, Exit);
@@ -763,6 +769,7 @@ OSString * AppleUSBAudioDevice::getNameForMixerPath (OSArray * arrayOfPathsFromO
 	UInt8								controlInterfaceNum;
 
 	string[0] = 0;
+	FailIf (NULL == mControlInterface, Exit);
 
 	aPath = OSDynamicCast (OSArray, arrayOfPathsFromOutputTerminal->getObject (*pathIndex));
 	FailIf (NULL == aPath, Exit);
@@ -828,6 +835,7 @@ void AppleUSBAudioDevice::addVolumeControls (AppleUSBAudioEngine * usbAudioEngin
 
 	engineInfoIndex = getEngineInfoIndex (usbAudioEngine);
 	FailIf (-1 == engineInfoIndex, Exit);
+	FailIf (NULL == mControlInterface, Exit);
 
 	engineInfo = OSDynamicCast (OSDictionary, mRegisteredEngines->getObject (engineInfoIndex));
 	FailIf (NULL == engineInfo, Exit);
@@ -922,6 +930,7 @@ void AppleUSBAudioDevice::addMuteControl (AppleUSBAudioEngine * usbAudioEngine,
 
 	engineInfoIndex = getEngineInfoIndex (usbAudioEngine);
 	FailIf (-1 == engineInfoIndex, Exit);
+	FailIf (NULL == mControlInterface, Exit);
 
 	engineInfo = OSDynamicCast (OSDictionary, mRegisteredEngines->getObject (engineInfoIndex));
 	FailIf (NULL == engineInfo, Exit);
@@ -985,10 +994,11 @@ SInt16 AppleUSBAudioDevice::getFeatureUnitSetting (UInt8 controlSelector, UInt8
     IOUSBDevRequestDesc					devReq;
 	IOReturn							result;
 	UInt16								theSetting;
-	IOBufferMemoryDescriptor *			theSettingDesc;
+	IOBufferMemoryDescriptor *			theSettingDesc = NULL;
 	UInt8								length;
 
 	theSetting = 0;
+	FailIf (NULL == mControlInterface, Exit);
 
 	switch (controlSelector) {
 		case MUTE_CONTROL:
@@ -1023,10 +1033,12 @@ Exit:
 
 IOReturn AppleUSBAudioDevice::setFeatureUnitSetting (UInt8 controlSelector, UInt8 unitID, UInt8 channelNumber, UInt8 requestType, UInt16 newValue, UInt16 newValueLen) {
     IOUSBDevRequestDesc					devReq;
-	IOBufferMemoryDescriptor *			theSettingDesc;
+	IOBufferMemoryDescriptor *			theSettingDesc = NULL;
 	IOReturn							result;
 
 	result = kIOReturnError;
+	FailIf (NULL == mControlInterface, Exit);
+
 	theSettingDesc = OSTypeAlloc (IOBufferMemoryDescriptor);
 	FailIf (NULL == theSettingDesc, Exit);
 	theSettingDesc->initWithBytes (&newValue, newValueLen, kIODirectionIn);
@@ -1298,6 +1310,7 @@ OSArray * AppleUSBAudioDevice::getPlaythroughPaths () {
 	UInt8								unitID;
 
 	playThroughPaths = NULL;
+	FailIf (NULL == mControlInterface, Exit);
 	controlInterfaceNum = mControlInterface->GetInterfaceNumber ();
 
 	numOutputTerminalArrays = mControlGraph->getCount ();
@@ -1346,6 +1359,7 @@ UInt8 AppleUSBAudioDevice::getBestFeatureUnitInPath (OSArray * thePath, UInt32 d
 	Boolean								found;
 
 	featureUnitID = 0;
+	FailIf (NULL == mControlInterface, Exit);
 	controlInterfaceNum = mControlInterface->GetInterfaceNumber ();
 	numUnitsInPath = thePath->getCount ();
 	found = FALSE;
@@ -1411,6 +1425,7 @@ UInt8 AppleUSBAudioDevice::getBestFeatureUnitInPath (OSArray * thePath, UInt32 d
 			break;
 	}
 
+Exit:
 	return featureUnitID;
 }
 
@@ -1418,9 +1433,11 @@ UInt8 AppleUSBAudioDevice::getSelectorSetting (UInt8 selectorID) {
     IOUSBDevRequestDesc					devReq;
 	IOReturn							result;
 	UInt8								setting;
-	IOBufferMemoryDescriptor *			settingDesc;
+	IOBufferMemoryDescriptor *			settingDesc = NULL;
 
 	setting = 0;
+	FailIf (NULL == mControlInterface, Exit);
+
 	settingDesc = IOBufferMemoryDescriptor::withOptions (kIODirectionIn, 1);
 	FailIf (NULL == settingDesc, Exit);
 
@@ -1445,9 +1462,11 @@ Exit:
 IOReturn AppleUSBAudioDevice::setSelectorSetting (UInt8 selectorID, UInt8 setting) {
     IOUSBDevRequestDesc					devReq;
 	IOReturn							result;
-	IOBufferMemoryDescriptor *			settingDesc;
+	IOBufferMemoryDescriptor *			settingDesc = NULL;
 
 	result = kIOReturnError;
+	FailIf (NULL == mControlInterface, Exit);
+
 	settingDesc = OSTypeAlloc (IOBufferMemoryDescriptor);
 	FailIf (NULL == settingDesc, Exit);
 	settingDesc->initWithBytes (&setting, 1, kIODirectionIn);
@@ -1475,7 +1494,6 @@ IOReturn AppleUSBAudioDevice::createControlsForInterface (IOAudioEngine *audioEn
 
     result = kIOReturnError;
 	mTerminatingDriver = FALSE;
-	FailIf (NULL == mControlInterface, Exit);
 
     usbAudioEngine = OSDynamicCast (AppleUSBAudioEngine, audioEngine);
     FailIf (NULL == usbAudioEngine, Exit);
@@ -1908,7 +1926,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequestDesc * request, IOUS
 	UInt32							timeout;
 	Boolean							done;
 
-	debugIOLog ("+AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", this, request);
+	debugIOLog ("+AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", this, request, completion);
 	result = kIOReturnSuccess;
 	FailIf (NULL == mInterfaceLock, Exit);
 	IORecursiveLockLock (mInterfaceLock);
@@ -1918,8 +1936,9 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequestDesc * request, IOUS
 		timeout = 5;
 		while (!done && timeout && mControlInterface) {
 			result = mControlInterface->DeviceRequest (request, completion);
-			if (result == kIOReturnAborted) {
+			if (result != kIOReturnSuccess) {
 				timeout--;
+				IOSleep (1);
 			} else {
 				done = TRUE;
 			}
@@ -1927,7 +1946,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequestDesc * request, IOUS
     }
 	IORecursiveLockUnlock (mInterfaceLock);
 
-	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", this, request);
+	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", this, request, completion, result);
 
 Exit:
 	return result;
@@ -1938,7 +1957,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest * request, IOUSBCom
 	UInt32							timeout;
 	Boolean							done;
 
-	debugIOLog ("+AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", this, request);
+	debugIOLog ("+AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", this, request, completion);
 	result = kIOReturnSuccess;
 	FailIf (NULL == mInterfaceLock, Exit);
 	IORecursiveLockLock (mInterfaceLock);
@@ -1948,8 +1967,9 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest * request, IOUSBCom
 		timeout = 5;
 		while (!done && timeout && mControlInterface) {
 			result = mControlInterface->DeviceRequest (request, completion);
-			if (result == kIOReturnAborted) {
+			if (result != kIOReturnSuccess) {
 				timeout--;
+				IOSleep (1);
 			} else {
 				done = TRUE;
 			}
@@ -1957,7 +1977,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest * request, IOUSBCom
     }
 	IORecursiveLockUnlock (mInterfaceLock);
 
-	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", this, request);
+	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", this, request, completion, result);
 
 Exit:
 	return result;
@@ -1978,8 +1998,9 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest *request, AppleUSBA
 		timeout = 5;
 		while (!done && timeout && self->mControlInterface) {
 			result = self->mControlInterface->DeviceRequest (request, completion);
-			if (result == kIOReturnAborted) {
+			if (result != kIOReturnSuccess) {
 				timeout--;
+				IOSleep (1);
 			} else {
 				done = TRUE;
 			}
@@ -1987,7 +2008,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest *request, AppleUSBA
 	}
 	IORecursiveLockUnlock (self->mInterfaceLock);
 
-	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)", self, request, completion);
+	debugIOLog ("-AppleUSBAudioDevice[%p]::deviceRequest (%p, %p) = %lx", self, request, completion, result);
 
 Exit:
 	return result;
