@@ -6,6 +6,9 @@
 #ifndef __USBAudioObject__
 #define __USBAudioObject__
 
+#define	kUSBAudioStreamInterfaceSubclass	2
+#define	kRootAlternateSetting				0
+
 enum {
 	kMuteBit					= 0,
 	kVolumeBit					= 1,
@@ -369,6 +372,23 @@ typedef struct ACProcessingUnitDescriptor {
 	// UInt8								processSpecific[]	// no set size, calculate
 } ACProcessingUnitDescriptor, *ACProcessingUnitDescriptorPtr;
 
+typedef struct ACExtensionUnitDescriptor {
+    UInt8									bLength;
+    UInt8									bDescriptorType;
+    UInt8									bDescriptorSubtype;
+    UInt8									bUnitID;
+	UInt16									wExtensionCode;
+	UInt8									bNrInPins;
+	UInt8									baSourceID[1];	// there are bNrInPins of these
+	// Can't have a static structure to define these locations:
+	// UInt8								bNrChannels
+	// UInt16								wChannelConfig
+	// UInt8								iChannelNames
+	// UInt8								bControlSize
+	// UInt8								bmControls[]
+	// UInt8								iExtension
+} ACExtensionUnitDescriptor, *ACExtensionUnitDescriptorPtr;
+
 /*	From USB Device Class Definition for Audio Data Formats 2.4.1:
 	The Type III Format Type is identical to the Type I PCM Format Type, set up for two-channel 16-bit PCM data.
 	It therefore uses two audio subframes per audio frame.  The subframe size is two bytes and the bit resolution
@@ -463,13 +483,17 @@ class USBACDescriptorObject : public OSObject {
 private:
 	UInt8							unitID;
 	UInt8							sourceID;
+	UInt8							descriptorSubType;
 
 public:
     virtual void					free (void);
 
+	virtual UInt8					GetDescriptorSubType (void) {return descriptorSubType;}
+	virtual UInt8					GetNumInPins (void) {return 1;}
 	virtual UInt8					GetSourceID (void) {return sourceID;}
 	virtual UInt8					GetUnitID (void) {return unitID;}
 
+	virtual void					SetDescriptorSubType (UInt8 subType) {descriptorSubType = subType;}
 	virtual void					SetSourceID (UInt8 srcID) {sourceID = srcID;}
 	virtual void					SetUnitID (UInt8 termID) {unitID = termID;}
 
@@ -526,6 +550,7 @@ public:
 	UInt16							GetChannelConfig (void) {return channelConfig;}
 	UInt8							GetNumInPins (void) {return numInPins;}
 	UInt8							GetNumChannels (void) {return numChannels;}
+	UInt8 *							GetSources (void) {return baSourceIDs;}
 
 	void							InitControlsArray (UInt8 * channelConfig, UInt8 bmControlSize);
 	void							InitSourceIDs (UInt8 * baSrcIDs, UInt8 nrInPins);
@@ -547,6 +572,51 @@ public:
 	void							InitSourceIDs (UInt8 * baSrcIDs, UInt8 nrInPins);
 };
 
+class USBProcessingUnitObject : public USBACDescriptorObject {
+private:
+	UInt8 *							bmControls;			// you don't know the size of this, calculate it using bLength and numInPins
+	UInt8 *							baSourceIDs;		// there are numInPins of these
+	UInt16							processType;
+	UInt16							channelConfig;
+	UInt8							numInPins;
+	UInt8							controlSize;		// This is the calculated size of bmControls
+	UInt8							numChannels;
+
+public:
+	virtual void					free (void);
+
+	UInt8							GetNumInPins (void) {return numInPins;}
+	UInt8 *							GetSources (void) {return baSourceIDs;}
+
+	void							InitSourceIDs (UInt8 * baSrcIDs, UInt8 nrInPins);
+	void							InitControlsArray (UInt8 * bmCntrls, UInt8 bmControlSize);
+	void							SetChannelConfig (UInt16 chanConfig) {channelConfig = chanConfig;}
+	void							SetProcessType (UInt16 wProcessType) {processType = wProcessType;}
+	void							SetNumChannels (UInt8 numChan) {numChannels = numChan;}
+};
+
+class USBExtensionUnitObject : public USBACDescriptorObject {
+private:
+	UInt8 *							bmControls;			// you don't know the size of this, calculate it using bLength and numInPins
+	UInt8 *							baSourceIDs;		// there are numInPins of these
+	UInt16							extensionCode;
+	UInt16							channelConfig;
+	UInt8							numInPins;
+	UInt8							controlSize;		// This is the calculated size of bmControls
+	UInt8							numChannels;
+
+public:
+	virtual void					free (void);
+
+	UInt8							GetNumInPins (void) {return numInPins;}
+	UInt8 *							GetSources (void) {return baSourceIDs;}
+
+	void							InitControlsArray (UInt8 * channelConfig, UInt8 bmControlSize);
+	void							InitSourceIDs (UInt8 * baSrcIDs, UInt8 nrInPins);
+	void							SetChannelConfig (UInt16 chanConfig) {channelConfig = chanConfig;}
+	void							SetNumChannels (UInt8 numChan) {numChannels = numChan;}
+};
+
 class USBFeatureUnitObject : public USBACDescriptorObject {
 private:
 	UInt8 *							bmaControls;
@@ -578,6 +648,8 @@ private:
 	OSArray *						theFeatureUnits;
 	OSArray *						theMixerUnits;
 	OSArray *						theSelectorUnits;
+	OSArray *						theProcessingUnits;
+	OSArray *						theExtensionUnits;
     UInt16							adcVersion;			// in BCD
     UInt8							interfaceNumber;
     UInt8							alternateSetting;
@@ -596,6 +668,8 @@ private:
 	USBMixerUnitObject *			GetMixerObject (UInt8 unitID);
 	USBOutputTerminalObject *		GetOutputTerminalObject (UInt8 unitID);
 	USBSelectorUnitObject *			GetSelectorUnitObject (UInt8 unitID);
+	USBProcessingUnitObject *		GetProcessingUnitObject (UInt8 unitID);
+	USBExtensionUnitObject *		GetExtensionUnitObject (UInt8 unitID);
 	USBACDescriptorObject *			GetACDescriptorObject (UInt8 unitID);
 
     void							SetAlternateSetting (UInt8 altIntNumber) {alternateSetting = altIntNumber;}
@@ -610,6 +684,7 @@ public:
     virtual void					free (void);
     USBInterfaceDescriptorPtr		ParseACInterfaceDescriptor (USBInterfaceDescriptorPtr theInterfacePtr, UInt8 const currentInterface);
 
+	UInt8							GetNumControls (UInt8 featureUnitID);
 	Boolean							ChannelHasMuteControl (UInt8 featureUnitID, UInt8 channelNum);
 	Boolean							ChannelHasVolumeControl (UInt8 featureUnitID, UInt8 channelNum);
 	UInt16							GetADCVersion (void) {return adcVersion;}
@@ -633,11 +708,19 @@ public:
 	UInt8							GetNumEndpoints (void) {return numEndpoints;}
 	UInt8							GetNumInputTerminals (void);
 	UInt8							GetNumOutputTerminals (void);
+	UInt8							GetNumSelectorUnits (void);
 	UInt8							GetNumSelectorSources (UInt8 unitID);
+	UInt8							GetNumSources (UInt8 unitID);
 	UInt8							GetNumStreamInterfaces (void) {return numStreamInterfaces;}
 	UInt8 *							GetStreamInterfaceNumbers (void) {return streamInterfaceNumbers;}
 	UInt16							GetOutputTerminalType (UInt8 terminalID);
 	UInt8 *							GetSelectorSources (UInt8 unitID);
+	UInt8 *							GetMixerSources (UInt8 unitID);
+	UInt8 *							GetExtensionUnitSources (UInt8 unitID);
+	UInt8 *							GetProcessingUnitSources (UInt8 unitID);
+	UInt8							GetSourceID (UInt8 unitID);
+	UInt8 *							GetSourceIDs (UInt8 unitID);
+	UInt8							GetSubType (UInt8 unitID);
 	Boolean							MasterHasMuteControl (UInt8 featureUnitID);
 };
 
@@ -822,10 +905,12 @@ public:
 	UInt16							GetMaxBitRate (UInt8 interfaceNum, UInt8 altInterfaceNum);
     UInt8							GetNumAltStreamInterfaces (UInt8 interfaceNum);
     UInt8							GetNumChannels (UInt8 interfaceNum, UInt8 altInterfaceNum);
+	UInt8							GetNumControls (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 featureUnitID);
     UInt8							GetNumSampleRates (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt8							GetNumInputTerminals (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt8							GetNumOutputTerminals (UInt8 interfaceNum, UInt8 altInterfaceNum);
-	UInt8							GetNumSelectorSources (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID);
+	UInt8							GetNumSelectorUnits (UInt8 interfaceNum, UInt8 altInterfaceNum);
+	UInt8							GetNumSources (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID);
     UInt8							GetNumStreamInterfaces (void);
 	UInt16							GetOutputTerminalType (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 terminalID);
 	UInt16							GetSamplesPerFrame (UInt8 interfaceNum, UInt8 altInterfaceNum);
@@ -833,7 +918,11 @@ public:
     UInt8							GetSampleSize (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt8 *							GetSelectorSources (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID);
     UInt8							GetSubframeSize (UInt8 interfaceNum, UInt8 altInterfaceNum);
+	UInt8							GetSubType (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID);
+	UInt8							GetSourceID (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID);		// Used for units that have only one input source
+	UInt8 *							GetSourceIDs (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID);		// Used for units that have multiple input sources
     UInt8							GetTerminalLink (UInt8 interfaceNum, UInt8 altInterfaceNum);
+	UInt8							GetUnitType (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 terminalNum);
 	Boolean							IsocEndpointDoesMaxPacketsOnly (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt8							IsocEndpointGetLockDelay (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt8							IsocEndpointGetLockDelayUnits (UInt8 interfaceNum, UInt8 altInterfaceNum);
